@@ -1454,8 +1454,13 @@ public class PgConnection implements BaseConnection {
           statement.execute("IDENTIFY_SYSTEM");
           statement.close();
         } else {
-          if (checkConnectionQuery == null) {
-            checkConnectionQuery = prepareStatement("");
+          PreparedStatement checkConnectionQuery;
+          synchronized (this) {
+            checkConnectionQuery = this.checkConnectionQuery;
+            if (checkConnectionQuery == null) {
+              checkConnectionQuery = prepareStatement("");
+              this.checkConnectionQuery = checkConnectionQuery;
+            }
           }
           checkConnectionQuery.executeUpdate();
         }
