@@ -240,11 +240,13 @@ public class PgStatement implements Statement, BaseStatement {
 
   @Override
   public ResultSet executeQuery(String sql) throws SQLException {
-    if (!executeWithFlags(sql, 0)) {
-      throw new PSQLException(GT.tr("No results were returned by the query."), PSQLState.NO_DATA);
-    }
+    synchronized (this) {
+      if (!executeWithFlags(sql, 0)) {
+        throw new PSQLException(GT.tr("No results were returned by the query."), PSQLState.NO_DATA);
+      }
 
-    return getSingleResultSet();
+      return getSingleResultSet();
+    }
   }
 
   protected ResultSet getSingleResultSet() throws SQLException {
@@ -262,9 +264,11 @@ public class PgStatement implements Statement, BaseStatement {
 
   @Override
   public int executeUpdate(String sql) throws SQLException {
-    executeWithFlags(sql, QueryExecutor.QUERY_NO_RESULTS);
-    checkNoResultUpdate();
-    return getUpdateCount();
+    synchronized (this) {
+      executeWithFlags(sql, QueryExecutor.QUERY_NO_RESULTS);
+      checkNoResultUpdate();
+      return getUpdateCount();
+    }
   }
 
   protected final void checkNoResultUpdate() throws SQLException {
@@ -404,17 +408,19 @@ public class PgStatement implements Statement, BaseStatement {
   protected final void execute(CachedQuery cachedQuery,
       /* @Nullable */ ParameterList queryParameters, int flags)
       throws SQLException {
-    try {
-      executeInternal(cachedQuery, queryParameters, flags);
-    } catch (SQLException e) {
-      // Don't retry composite queries as it might get partially executed
-      if (cachedQuery.query.getSubqueries() != null
-          || !connection.getQueryExecutor().willHealOnRetry(e)) {
-        throw e;
+    synchronized (this) {
+      try {
+        executeInternal(cachedQuery, queryParameters, flags);
+      } catch (SQLException e) {
+        // Don't retry composite queries as it might get partially executed
+        if (cachedQuery.query.getSubqueries() != null
+            || !connection.getQueryExecutor().willHealOnRetry(e)) {
+          throw e;
+        }
+        cachedQuery.query.close();
+        // Execute the query one more time
+        executeInternal(cachedQuery, queryParameters, flags);
       }
-      cachedQuery.query.close();
-      // Execute the query one more time
-      executeInternal(cachedQuery, queryParameters, flags);
     }
   }
 
@@ -514,7 +520,10 @@ public class PgStatement implements Statement, BaseStatement {
     // No-op.
   }
 
-  private volatile boolean isClosed = false;
+  private volatile int isClosed = 0;
+  private static final AtomicIntegerFieldUpdater<PgStatement> IS_CLOSED_UPDATER =
+      AtomicIntegerFieldUpdater.newUpdater(
+          PgStatement.class, "isClosed");
 
   @Override
   public int getUpdateCount() throws SQLException {
@@ -667,11 +676,8 @@ public class PgStatement implements Statement, BaseStatement {
    */
   public final void close() throws SQLException {
     // closing an already closed Statement is a no-op.
-    synchronized (this) {
-      if (isClosed) {
-        return;
-      }
-      isClosed = true;
+    if (!IS_CLOSED_UPDATER.compareAndSet(this, 0, 1)) {
+      return;
     }
 
     cancel();
@@ -1090,9 +1096,11 @@ public class PgStatement implements Statement, BaseStatement {
 
   @Override
   public long executeLargeUpdate(String sql) throws SQLException {
-    executeWithFlags(sql, QueryExecutor.QUERY_NO_RESULTS);
-    checkNoResultUpdate();
-    return getLargeUpdateCount();
+    synchronized (this) {
+      executeWithFlags(sql, QueryExecutor.QUERY_NO_RESULTS);
+      checkNoResultUpdate();
+      return getLargeUpdateCount();
+    }
   }
 
   @Override
@@ -1116,19 +1124,21 @@ public class PgStatement implements Statement, BaseStatement {
 
   @Override
   public long executeLargeUpdate(String sql, String /* @Nullable */ [] columnNames) throws SQLException {
-    if (columnNames != null && columnNames.length == 0) {
-      return executeLargeUpdate(sql);
-    }
+    synchronized (this) {
+      if (columnNames != null && columnNames.length == 0) {
+        return executeLargeUpdate(sql);
+      }
 
-    wantsGeneratedKeysOnce = true;
-    if (!executeCachedSql(sql, 0, columnNames)) {
-      // no resultset returned. What's a pity!
+      wantsGeneratedKeysOnce = true;
+      if (!executeCachedSql(sql, 0, columnNames)) {
+        // no resultset returned. What's a pity!
+      }
+      return getLargeUpdateCount();
     }
-    return getLargeUpdateCount();
   }
 
   public boolean isClosed() throws SQLException {
-    return isClosed;
+    return isClosed == 1;
   }
 
   public void setPoolable(boolean poolable) throws SQLException {
@@ -1240,15 +1250,17 @@ public class PgStatement implements Statement, BaseStatement {
   }
 
   public int executeUpdate(String sql, String /* @Nullable */ [] columnNames) throws SQLException {
-    if (columnNames != null && columnNames.length == 0) {
-      return executeUpdate(sql);
-    }
+    synchronized (this) {
+      if (columnNames != null && columnNames.length == 0) {
+        return executeUpdate(sql);
+      }
 
-    wantsGeneratedKeysOnce = true;
-    if (!executeCachedSql(sql, 0, columnNames)) {
-      // no resultset returned. What's a pity!
+      wantsGeneratedKeysOnce = true;
+      if (!executeCachedSql(sql, 0, columnNames)) {
+        // no resultset returned. What's a pity!
+      }
+      return getUpdateCount();
     }
-    return getUpdateCount();
   }
 
   public boolean execute(String sql, int autoGeneratedKeys) throws SQLException {
@@ -1268,12 +1280,14 @@ public class PgStatement implements Statement, BaseStatement {
   }
 
   public boolean execute(String sql, String /* @Nullable */ [] columnNames) throws SQLException {
-    if (columnNames != null && columnNames.length == 0) {
-      return execute(sql);
-    }
+    synchronized (this) {
+      if (columnNames != null && columnNames.length == 0) {
+        return execute(sql);
+      }
 
-    wantsGeneratedKeysOnce = true;
-    return executeCachedSql(sql, 0, columnNames);
+      wantsGeneratedKeysOnce = true;
+      return executeCachedSql(sql, 0, columnNames);
+    }
   }
 
   public int getResultSetHoldability() throws SQLException {
