@@ -15,7 +15,6 @@ import static org.junit.Assert.fail;
 import org.postgresql.PGProperty;
 import org.postgresql.core.ServerVersion;
 import org.postgresql.jdbc.PgStatement;
-import org.postgresql.test.SlowTests;
 import org.postgresql.test.TestUtil;
 import org.postgresql.test.util.StrangeProxyServer;
 import org.postgresql.util.PSQLState;
@@ -25,7 +24,6 @@ import org.junit.Assert;
 import org.junit.Assume;
 import org.junit.Before;
 import org.junit.Test;
-import org.junit.experimental.categories.Category;
 
 import java.io.IOException;
 import java.sql.Connection;
@@ -34,7 +32,9 @@ import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.SQLWarning;
 import java.sql.Statement;
+import java.util.ArrayList;
 import java.util.HashMap;
+import java.util.List;
 import java.util.Map;
 import java.util.Properties;
 import java.util.Random;
@@ -697,7 +697,6 @@ public class StatementTest {
   }
 
   @Test
-  @Category(SlowTests.class)
   public void testSetQueryTimeout() throws SQLException {
     Statement stmt = con.createStatement();
     long start = 0;
@@ -720,7 +719,6 @@ public class StatementTest {
   }
 
   @Test
-  @Category(SlowTests.class)
   public void testLongQueryTimeout() throws SQLException {
     Statement stmt = con.createStatement();
     stmt.setQueryTimeout(Integer.MAX_VALUE);
@@ -736,7 +734,6 @@ public class StatementTest {
    * one does not. The timeout of the first query should not impact the second one.
    */
   @Test
-  @Category(SlowTests.class)
   public void testShortQueryTimeout() throws SQLException {
     assumeLongTest();
 
@@ -863,11 +860,10 @@ public class StatementTest {
   }
 
   @Test(timeout = 30000)
-  @Category(SlowTests.class)
   public void testCancelQueryWithBrokenNetwork() throws SQLException, IOException, InterruptedException {
     // check that stmt.cancel() doesn't hang forever if the network is broken
 
-    ExecutorService executor = Executors.newSingleThreadExecutor();
+    ExecutorService executor = Executors.newCachedThreadPool();
 
     try (StrangeProxyServer proxyServer = new StrangeProxyServer(TestUtil.getServer(), TestUtil.getPort())) {
       Properties props = new Properties();
@@ -881,6 +877,9 @@ public class StatementTest {
         proxyServer.stopForwardingAllClients();
 
         stmt.cancel();
+        // Note: network is still inaccessible, so the statement execution is still in progress.
+        // So we abort the connection to allow implicit conn.close()
+        conn.abort(executor);
       }
     }
 
@@ -945,6 +944,51 @@ public class StatementTest {
     }
   }
 
+  @Test(timeout = 10000)
+  public void testConcurrentIsValid() throws Throwable {
+    ExecutorService executor = Executors.newCachedThreadPool();
+    try {
+      List<Future<?>> results = new ArrayList<>();
+      Random rnd = new Random();
+      for (int i = 0; i < 10; i++) {
+        Future<?> future = executor.submit(() -> {
+          try {
+            for (int j = 0; j < 50; j++) {
+              con.isValid(1);
+              try (PreparedStatement ps =
+                       con.prepareStatement("select * from generate_series(1,?) as x(id)")) {
+                int limit = rnd.nextInt(10);
+                ps.setInt(1, limit);
+                try (ResultSet r = ps.executeQuery()) {
+                  int cnt = 0;
+                  String callName = "generate_series(1, " + limit + ") in thread "
+                      + Thread.currentThread().getName();
+                  while (r.next()) {
+                    cnt++;
+                    assertEquals(callName + ", row " + cnt, cnt, r.getInt(1));
+                  }
+                  assertEquals(callName + " number of rows", limit, cnt);
+                }
+              }
+            }
+          } catch (SQLException e) {
+            throw new RuntimeException(e);
+          }
+        });
+        results.add(future);
+      }
+      for (Future<?> result : results) {
+        // Propagate exception if any
+        result.get();
+      }
+    } catch (ExecutionException e) {
+      throw e.getCause();
+    } finally {
+      executor.shutdown();
+      executor.awaitTermination(10, TimeUnit.SECONDS);
+    }
+  }
+
   @Test(timeout = 20000)
   public void testFastCloses() throws SQLException {
     ExecutorService executor = Executors.newSingleThreadExecutor();
